import { DataObject, AbstractClass, IDType, NewData } from './utility';
import { BaseListModel } from './Base';
export interface PageData<D extends DataObject> {
    pageData: D[];
    totalCount?: number;
}
export declare abstract class ListModel<D extends DataObject, F = NewData<D>> extends BaseListModel<D> {
    pageIndex: number;
    pageSize: number;
    filter: F;
    totalCount: number;
    pageList: D[][];
    get currentPage(): D[];
    get pageCount(): number;
    get allItems(): D[];
    get noMore(): boolean;
    constructor();
    clearList(): void;
    clear(): void;
    restoreList({ pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D>, 'pageSize' | 'allItems' | 'totalCount'>>): void;
    turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
    abstract loadPage(pageIndex: number, pageSize: number, filter: F): Promise<PageData<D>>;
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    refreshList(): Promise<D[]>;
    indexOf(id: IDType): number;
    changeOne(data: Partial<D>, id: IDType, patch?: boolean): void;
    updateOne(data: NewData<D>, id?: IDType): Promise<D>;
    removeOne(id: IDType): Promise<void>;
    deleteOne(id: IDType): Promise<void>;
}
export declare function Buffer<D extends DataObject, F = NewData<D>, M extends AbstractClass<ListModel<D, F>> = AbstractClass<ListModel<D, F>>>(Super: M): (abstract new (...data: any[]) => {
    pendingList: ReturnType<ListModel<D, F>['loadPage']>[];
    clearList(): void;
    clear(): void;
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    pageIndex: number;
    pageSize: number;
    filter: F;
    totalCount: number;
    pageList: D[][];
    readonly currentPage: D[];
    readonly pageCount: number;
    readonly allItems: D[];
    readonly noMore: boolean;
    restoreList({ pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D, Partial<Omit<D, import("web-utility").TypeKeys<D, DataObject> | import("web-utility").TypeKeys<D, DataObject[]>> & { [K in import("web-utility").TypeKeys<D, DataObject>]: IDType; } & { [K_1 in import("web-utility").TypeKeys<D, DataObject[]>]: IDType[]; }>>, "pageSize" | "totalCount" | "allItems">>): void;
    turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
    loadPage(pageIndex: number, pageSize: number, filter: F): Promise<PageData<D>>;
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    refreshList(): Promise<D[]>;
    indexOf(id: IDType): number;
    changeOne(data: Partial<D>, id: IDType, patch?: boolean): void;
    updateOne(data: Partial<Omit<D, import("web-utility").TypeKeys<D, DataObject> | import("web-utility").TypeKeys<D, DataObject[]>> & { [K in import("web-utility").TypeKeys<D, DataObject>]: IDType; } & { [K_1 in import("web-utility").TypeKeys<D, DataObject[]>]: IDType[]; }>, id?: IDType): Promise<D>;
    removeOne(id: IDType): Promise<void>;
    deleteOne(id: IDType): Promise<void>;
    client: import("./utility").RESTClient;
    baseURI: string;
    indexKey: keyof D;
    currentOne: D;
    validity: Partial<Record<keyof D, {
        [type: string]: string;
    }>>;
    clearCurrent(): void;
    getOne(id: IDType): Promise<D>;
    downloading: number;
    uploading: number;
}) & M;
export declare function Stream<D extends DataObject, F = NewData<D>, M extends AbstractClass<ListModel<D, F>> = AbstractClass<ListModel<D, F>>>(Super: M): (abstract new (...data: any[]) => {
    baseURI: string;
    stream?: AsyncGenerator<D, void, any>;
    openStream(filter: F): AsyncGenerator<D, void, any>;
    clearList(): void;
    clear(): void;
    restoreList({ filter, pageSize, allItems, totalCount }?: Partial<Pick<ListModel<D>, 'filter' | 'pageSize' | 'allItems' | 'totalCount'>>): Promise<void>;
    loadStream(filter: F, newCount: number): Promise<D[]>;
    loadPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    pageIndex: number;
    pageSize: number;
    filter: F;
    totalCount: number;
    pageList: D[][];
    readonly currentPage: D[];
    readonly pageCount: number;
    readonly allItems: D[];
    readonly noMore: boolean;
    turnTo(pageIndex: number, pageSize?: number): ListModel<D, F>;
    loadNewPage(pageIndex: number, pageSize: number, filter: F): Promise<{
        pageData: D[];
        totalCount: number;
    }>;
    getList(filter?: F, pageIndex?: number, pageSize?: number): Promise<D[]>;
    refreshList(): Promise<D[]>;
    indexOf(id: IDType): number;
    changeOne(data: Partial<D>, id: IDType, patch?: boolean): void;
    updateOne(data: Partial<Omit<D, import("web-utility").TypeKeys<D, DataObject> | import("web-utility").TypeKeys<D, DataObject[]>> & { [K in import("web-utility").TypeKeys<D, DataObject>]: IDType; } & { [K_1 in import("web-utility").TypeKeys<D, DataObject[]>]: IDType[]; }>, id?: IDType): Promise<D>;
    removeOne(id: IDType): Promise<void>;
    deleteOne(id: IDType): Promise<void>;
    client: import("./utility").RESTClient;
    indexKey: keyof D;
    currentOne: D;
    validity: Partial<Record<keyof D, {
        [type: string]: string;
    }>>;
    clearCurrent(): void;
    getOne(id: IDType): Promise<D>;
    downloading: number;
    uploading: number;
}) & M;
